@@ -1,26 +1,26 @@
-#' Calls fastPHASE to fit an HMM to genotype data
-#' 
+#' Fit an HMM to genetic data using fastPHASE
+#'
 #' This function provides a wrapper for the fastPHASE executable in order to fit an HMM to either
-#' unphased genotype data or phased haplotype data. 
-#' The software fastPHASE will fit the HMM  to the genotype data and write the 
-#' corresponding parameter estimates in four separate files. 
-#' Since fastPHASE is not an R package, this executable must be downloaded separately by the 
+#' unphased genotype data or phased haplotype data.
+#' The software fastPHASE will fit the HMM  to the genotype data and write the
+#' corresponding parameter estimates in four separate files.
+#' Since fastPHASE is not an R package, this executable must be downloaded separately by the
 #' user. Visit \url{http://scheet.org/software.html} for more information on how to obtain fastPHASE.
-#' 
+#'
 #' @param fp_path a string with the path to the directory with the fastPHASE executable.
-#' @param X_file a string with the path of the genotype input file containing X in fastPHASE  
-#'               format (as created by \link{SNPknock.fp.writeX}).
-#' @param out_path a string with the path of the directory in which the parameter estimates 
-#'                 will be saved (default: NULL). If this is equal to NULL, a temporary file 
+#' @param X_file a string with the path of the genotype input file containing X in fastPHASE
+#'               format (as created by \link{writeXtoInp}).
+#' @param out_path a string with the path of the directory in which the parameter estimates
+#'                 will be saved (default: NULL). If this is equal to NULL, a temporary file
 #'                 in the R temporary directory will be used.
 #' @param K the number of hidden states for each haplotype sequence (default: 12).
 #' @param numit the number of EM iterations (default: 25).
 #' @param phased whether the data are already phased (default: FALSE).
 #' @param seed the random seed for the EM algorithm (default: 1).
-#' @return A string containing the path of the directory in which the parameter estimates 
-#'         were saved. This is useful to find the data when the default option for `out_path` 
+#' @return A string containing the path of the directory in which the parameter estimates
+#'         were saved. This is useful to find the data when the default option for `out_path`
 #'         is used and the output is written in an R temporary directory.
-#' 
+#'
 #' @family fastPHASE
 #'
 #' @details
@@ -32,31 +32,29 @@
 #'   \item{"_thetahat.txt"}
 #'   \item{"_origchars"}
 #' }
-#' 
+#'
 #' The HMM for the genotype data can then be loaded from these files by calling
-#' \link{SNPknock.fp.loadFit}.
-#' 
-#' @references 
-#'   Scheet and Stephens,  A fast and flexible statistical model for large-scale population genotype data,
-#'   Am J Hum Genet (2006).
-#'   \href{http://www.sciencedirect.com/science/article/pii/S000292970763701X}{http://www.sciencedirect.com/science/article/pii/S000292970763701X}
-#' 
+#' \link{loadHMM}.
+#'
+#' @references
+#'   \insertRef{scheet2006}{SNPknock}
+#'   
 #' @examples
 #' fp_path  = "~/bin/fastPHASE" # Path to the fastPHASE executable
-#' 
+#'
 #' # Run fastPHASE on unphased genotypes
 #' # Specify the path to the genotype input file in ".inp" format.
 #' # An example file containing unphased genotypes can be found in the package installation folder.
 #' X_file = system.file("extdata", "genotypes.inp", package = "SNPknock")
-#' fp_outPath = SNPknock.fp.runFastPhase(fp_path, X_file)
-#' 
+#' fp_outPath = runFastPhase(fp_path, X_file)
+#'
 #' # Run fastPHASE on phased haplotypes
 #' # An example file containing phased haplotypes can be found in the package installation folder.
 #' H_file = system.file("extdata", "haplotypes.inp", package = "SNPknock")
-#' fp_outPath = SNPknock.fp.runFastPhase(fp_path, H_file, phased=TRUE)
-#' 
+#' fp_outPath = runFastPhase(fp_path, H_file, phased=TRUE)
+#'
 #' @export
-SNPknock.fp.runFastPhase <- function(fp_path, X_file, out_path=NULL, K=12, numit=25, phased=FALSE, seed=1) {
+runFastPhase <- function(fp_path, X_file, out_path=NULL, K=12, numit=25, phased=FALSE, seed=1) {
   # Check that input has the right format
   K = as.integer(K)
   numit = as.integer(numit)
@@ -68,24 +66,24 @@ SNPknock.fp.runFastPhase <- function(fp_path, X_file, out_path=NULL, K=12, numit
   stopifnot(is.integer(numit))
   stopifnot(is.logical(phased))
   stopifnot(is.integer(seed))
-  
+
   # Verify that the fastPHASE executable can be found
   if(!file.exists(fp_path)) {
     message(paste("SNPknock could find the fastPHASE executable: '",fp_path,"' does not exist.
 If you have not downloaded it yet, you can obtain fastPHASE from: http://scheet.org/software.html", sep=""))
-    return(NULL)  
+    return(NULL)
   }
-  
+
   # Write to temporary directory unless specified otherwise
   if(is.null(out_path)) {
     out_path = tempfile(pattern="file", tmpdir=tempdir(), fileext = "")
   }
-  
+
   # Make out_path absolute
   out_path_dirname = tools::file_path_as_absolute(dirname(out_path))
   out_path_basename = basename(out_path)
   out_path_abs = paste(out_path_dirname, out_path_basename, sep="/")
-  
+
   # Prepare arguments for fastPHASE
   command = fp_path
   command = paste(command, " -Pp -T1 -K", K, sep="")
@@ -95,57 +93,56 @@ If you have not downloaded it yet, you can obtain fastPHASE from: http://scheet.
   }
   command = paste(command, " -S", seed, sep="")
   command = paste(command, " -o'", out_path_abs, "' ", X_file, sep="")
-  
+
   # Run the fastPHASE executable
+  cat(command)
   tryCatch(system(command), error=function(e) 1)
-  
+
   return(out_path)
 }
 
-#' Convert a genetic matrix X into the fastPHASE input format
-#' 
-#' This function convert a genetic matrix X into the fastPHASE input format and saves
+#' Convert genotypes X into the fastPHASE input format
+#'
+#' This function converts a genetic matrix X into the fastPHASE input format and saves
 #' it to a user-specified file. Then, an HMM can be fitted by calling fastPHASE with
-#' \link{SNPknock.fp.runFastPhase}.
-#' 
-#' @param X either a matrix of size n-by-p containing unphased genotypes for n individuals, 
+#' \link{runFastPhase}.
+#'
+#' @param X either a matrix of size n-by-p containing unphased genotypes for n individuals,
 #' or a matrix of size 2n-by-p containing phased haplotypes for n individuals.
 #' @param phased whether the data are phased (default: FALSE).
-#' If this is equal to TRUE, each pair of consecutive rows will be assumed to correspond to phased haplotypes from the 
+#' If this is equal to TRUE, each pair of consecutive rows will be assumed to correspond to phased haplotypes from the
 #' same individual.
-#' @param out_file a string containing the path of the output file onto which X will be written (default: NULL). 
+#' @param out_file a string containing the path of the output file onto which X will be written (default: NULL).
 #' If this is equal to NULL, a temporary file in the R temporary directory will be used.
-#' @return A string containing the path of the output file onto which X was written. This is useful to find the data 
+#' @return A string containing the path of the output file onto which X was written. This is useful to find the data
 #' when the default option for `out_file` is used and X is written onto a temporary file in the R temporary directory.
-#' 
+#'
 #' @family fastPHASE
-#' 
-#' @references 
-#'   Scheet and Stephens,  A fast and flexible statistical model for large-scale population genotype data,
-#'   Am J Hum Genet (2006).
-#'   \href{http://www.sciencedirect.com/science/article/pii/S000292970763701X}{http://www.sciencedirect.com/science/article/pii/S000292970763701X}
-#' 
+#'
+#' @references
+#'   \insertRef{scheet2006}{SNPknock}
+#'
 #' @examples
 #' # Convert unphased genotypes
 #' # Load an example data matrix X from the package installation directory.
 #' X_file = system.file("extdata", "genotypes.RData", package = "SNPknock")
 #' load(X_file)
 #' # Write X in a temporary file
-#' Xinp_file = SNPknock.fp.writeX(X)
-#' 
+#' Xinp_file = writeXtoInp(X)
+#'
 #' # Convert phased haplotypes
 #' # Load an example data matrix H from the package installation directory.
 #' H_file = system.file("extdata", "haplotypes.RData", package = "SNPknock")
 #' load(H_file)
 #' # Write H in a temporary file
-#' Hinp_file = SNPknock.fp.writeX(H, phased=TRUE)
-#' 
+#' Hinp_file = writeXtoInp(H, phased=TRUE)
+#'
 #' @export
-SNPknock.fp.writeX <- function(X, phased=FALSE, out_file=NULL) {
+writeXtoInp <- function(X, phased=FALSE, out_file=NULL) {
   # Extract dimensions of input
   n = dim(X)[1]
   p = dim(X)[2]
-  
+
   # Check that input has the right format
   stopifnot(is.integer(X))
   stopifnot(is.logical(phased))
@@ -158,12 +155,12 @@ SNPknock.fp.writeX <- function(X, phased=FALSE, out_file=NULL) {
     stopifnot(min(X)>=0)
     stopifnot(max(X)<=2)
   }
-  
+
   # Write to temporary file unless specified otherwise
   if(is.null(out_file)) {
     out_file = tempfile(pattern="file", tmpdir=tempdir(), fileext = ".inp")
   }
-  
+
   # Write header to file
   con = file(out_file, "w")
   if(phased) {
@@ -173,7 +170,7 @@ SNPknock.fp.writeX <- function(X, phased=FALSE, out_file=NULL) {
     writeLines(toString(n), con = con, sep = "\n", useBytes = FALSE)
   }
   writeLines(toString(p), con = con, sep = "\n", useBytes = FALSE)
-  
+
   # Write data to file
   if(phased){
     # Write phased haplotypes to file
@@ -202,40 +199,43 @@ SNPknock.fp.writeX <- function(X, phased=FALSE, out_file=NULL) {
       writeLines(text, con = con, sep = "\n", useBytes = FALSE)
     }
   }
-  
+
   close(con)
-  
+
   return(out_file)
 }
 
-#' Load the parameter estimates obtained by fastPHASE
-#' 
-#' This function loads the parameter estimates obtained by fastPHASE (see \link{SNPknock.fp.runFastPhase})
+#' Load HMM parameters fitted by fastPHASE
+#'
+#' This function loads the parameter estimates obtained by fastPHASE (see \link{runFastPhase})
 #' and assembles the Li and Stephens HMM, in the format required by the knockoff generation functions
-#' \link{SNPknock.knockoffHaplotypes} and \link{SNPknock.knockoffGenotypes}.
-#' 
+#' \link{knockoffHaplotypes} and \link{knockoffGenotypes}.
+#'
 #' @param r_file a string with the path of the "_rhat.txt" file produced by fastPHASE.
 #' @param alpha_file a string with the path of the "_alphahat.txt" file produced by fastPHASE.
 #' @param theta_file a string with the path of the "_thetahat.txt" file produced by fastPHASE.
 #' @param char_file a string with the path of the "_origchars" file produced by fastPHASE.
-#' 
+#' @param compact whether to assemble the explicit transition and emission matrices for the HMM (default: FALSE).
+#' @param phased whether to assemble a model for phased haplotypes, if compact==FALSE (default: FALSE).
+#'
 #' @return A structure containing the parameters from the Li and Stephens HMM for phased haplotypes.
-#' 
+#'
 #' @family fastPHASE
-#' 
+#'
 #' @details
-#' This function returns a structure with three fields: 
+#' This function by default returns a structure with three fields:
 #' \itemize{
 #'   \item{"r": a numerical array of length p.}
 #'   \item{"alpha": a numerical array of size (p,K).}
 #'   \item{"theta": a numerical array of size (p,K).}
 #'  }
-#'   
-#' @references 
-#'   Scheet and Stephens,  A fast and flexible statistical model for large-scale population genotype data,
-#'   Am J Hum Genet (2006).
-#'   \href{http://www.sciencedirect.com/science/article/pii/S000292970763701X}{http://www.sciencedirect.com/science/article/pii/S000292970763701X}
+#'
+#' If the parameter compact is FALSE, this function assembles the HMM model for the genotype data
+#' (either unphased or phased), in the format required by the knockoff generation function \link{knockoffHMM}.
 #' 
+#' @references
+#'   \insertRef{scheet2006}{SNPknock}
+#'
 #' @examples
 #' # Specify the location of the fastPHASE output files containing the parameter estimates.
 #' # Example files can be found in the package installation directory.
@@ -243,112 +243,55 @@ SNPknock.fp.writeX <- function(X, phased=FALSE, out_file=NULL) {
 #' alpha_file = system.file("extdata", "genotypes_alphahat.txt", package = "SNPknock")
 #' theta_file = system.file("extdata", "genotypes_thetahat.txt", package = "SNPknock")
 #' char_file = system.file("extdata", "genotypes_origchars", package = "SNPknock")
-#' 
-#' # Read the parameter files and build the HMM
-#' hmm = SNPknock.fp.loadFit(r_file, alpha_file, theta_file, char_file)
-#' 
+#'
+#' # Read the parameter files and load the HMM
+#' hmm = loadHMM(r_file, alpha_file, theta_file, char_file)
+#'
+#' # Read the parameter files and load the HMM
+#' hmm.large = loadHMM(r_file, alpha_file, theta_file, char_file, compact=FALSE)
+#'
 #' @export
-SNPknock.fp.loadFit <- function(r_file, alpha_file, theta_file, char_file) {
+loadHMM <- function(r_file, alpha_file, theta_file, char_file, compact=TRUE, phased=FALSE) {
   # Check that input has the right format
   stopifnot(is.character(r_file))
   stopifnot(is.character(alpha_file))
   stopifnot(is.character(theta_file))
   stopifnot(is.character(char_file))
-  
-  # Load (r,theta,alpha) paramters from fastPHASE fit
-  r = loadEMParameters(r_file)
-  alpha = loadEMParameters(alpha_file)
-  theta = loadEMParameters(theta_file)
-  char = loadEMParameters(char_file)
-  
-  # Flip theta
-  X_chr_flip = char[,2] %in% c("1?", "10")
-  theta[X_chr_flip,] = 1-theta[X_chr_flip,]
-  
-  hmm = NULL
-  hmm$r = r
-  hmm$alpha = alpha
-  hmm$theta = theta
-  return(hmm) 
-}
 
-#' Load the parameter estimates obtained by fastPHASE and assembles the HMM model for genotype data
-#' 
-#' This function loads the parameter estimates obtained by fastPHASE (see \link{SNPknock.fp.runFastPhase})
-#' and assembles the HMM model for the genotype data (either unphased or phased), 
-#' in the format required by the knockoff generation function \link{SNPknock.knockoffHMM}.
-#' 
-#' @param r_file a string with the path of the "_rhat.txt" file produced by fastPHASE.
-#' @param alpha_file a string with the path of the "_alphahat.txt" file produced by fastPHASE.
-#' @param theta_file a string with the path of the "_thetahat.txt" file produced by fastPHASE.
-#' @param char_file a string with the path of the "_origchars" file produced by fastPHASE.
-#' @param phased whether to assemble a model for phased haplotypes (default: FALSE).
-#' 
-#' @return A structure describing the HMM fitted by fastPHASE.
-#' 
-#' @family fastPHASE
-#' 
-#' @details
-#' This function returns a structure with three fields: 
-#' \itemize{
-#'   \item{"pInit": a numerical array of length K, containing the marginal distribution of the hidden states for the first SNP.}
-#'   \item{"Q": a numerical array of size (p-1,K,K), containing a list of p-1 transition matrices between the K latent states of the HMM.}
-#'   \item{"pEmit": a numerical array of size (p,K,3), containing the emission probabilities of the hidden states for each of the p SNPs.}
-#'  }
-#'   
-#' @references 
-#'   Scheet and Stephens,  A fast and flexible statistical model for large-scale population genotype data,
-#'   Am J Hum Genet (2006).
-#'   \href{http://www.sciencedirect.com/science/article/pii/S000292970763701X}{http://www.sciencedirect.com/science/article/pii/S000292970763701X}
-#' 
-#' @examples
-#' # Specify the location of the fastPHASE output files containing the parameter estimates.
-#' # Example files can be found in the package installation directory.
-#' r_file = system.file("extdata", "genotypes_rhat.txt", package = "SNPknock")
-#' alpha_file = system.file("extdata", "genotypes_alphahat.txt", package = "SNPknock")
-#' theta_file = system.file("extdata", "genotypes_thetahat.txt", package = "SNPknock")
-#' char_file = system.file("extdata", "genotypes_origchars", package = "SNPknock")
-#' 
-#' # Read the parameter files and build the HMM for unphased genotypes
-#' hmm = SNPknock.fp.loadFit_hmm(r_file, alpha_file, theta_file, char_file)
-#' 
-#' # Read the parameter files and build the HMM for phased haplotypes
-#' hmm = SNPknock.fp.loadFit_hmm(r_file, alpha_file, theta_file, char_file, phased=TRUE)
-#' 
-#' @export
-SNPknock.fp.loadFit_hmm <- function(r_file, alpha_file, theta_file, char_file, phased=FALSE) {
   # Load (r,theta,alpha) paramters from fastPHASE fit
   r = loadEMParameters(r_file)
   alpha = loadEMParameters(alpha_file)
   theta = loadEMParameters(theta_file)
   char = loadEMParameters(char_file)
-  
+
   # Flip theta
   X_chr_flip = char[,2] %in% c("1?", "10")
   theta[X_chr_flip,] = 1-theta[X_chr_flip,]
-  
-  # Assemble transition matrices
-  if(phased) {
-    Q = compute_Q1(r, alpha)
-    pEmit = assemble_pEmit_phased(theta)
-    pInit = assemble_pInit_phased(alpha)
-    
-  } else {
-    Q1 = compute_Q1(r, alpha)
-    Q = assemble_Q(Q1)
-    pEmit = assemble_pEmit(theta)
-    pInit = assemble_pInit(alpha)
-  }
-  
+
+  # Return the HMM
   hmm = NULL
-  hmm$pInit = pInit
-  hmm$Q = Q
-  hmm$pEmit = pEmit
-  return(hmm) 
+  if(compact) {
+      hmm$r = r
+      hmm$alpha = alpha
+      hmm$theta = theta
+  } else {
+    # Assemble transition matrices
+    if(phased) {
+        hmm$Q = compute_Q1(r, alpha)
+        hmm$pEmit = assemble_pEmit_phased(theta)
+        hmm$pInit = assemble_pInit_phased(alpha)
+    } else {
+        Q1 = compute_Q1(r, alpha)
+        hmm$Q = assemble_Q(Q1)
+        hmm$pEmit = assemble_pEmit(theta)
+        hmm$pInit = assemble_pInit(alpha)
+    }
+  }
+  return(hmm)
 }
 
 #' Read the files produced by fastPHASE
-#'  
+#'
 #' @rdname loadEMParameters
 #' @keywords internal
 loadEMParameters <- function(data_path) {
@@ -367,7 +310,7 @@ loadEMParameters <- function(data_path) {
 }
 
 #' Compute the haplotype transition matrices based on the fastPHASE HMM
-#'  
+#'
 #' @rdname compute_Q1
 #' @keywords internal
 compute_Q1 <- function(r, alpha) {
@@ -383,7 +326,7 @@ compute_Q1 <- function(r, alpha) {
 }
 
 #' Compute the genotype transition matrices based on the fastPHASE HMM
-#'  
+#'
 #' @rdname assemble_Q
 #' @keywords internal
 assemble_Q <- function(Q1) {
@@ -408,14 +351,14 @@ assemble_Q <- function(Q1) {
   for(m in 1:(p-1)) {
     # Enforce normalization
     QSums = rowSums(Q[m,,])
-    stopifnot(abs(QSums-1)<1e-6)
+    #stopifnot(abs(QSums-1)<1e-6)
     Q[m,,] = sweep(Q[m,,],1,QSums,`/`)
   }
   return(Q)
 }
 
 #' Compute the genotype emission distributions based on the fastPHASE HMM
-#'  
+#'
 #' @rdname assemble_pEmit
 #' @keywords internal
 assemble_pEmit<- function(theta) {
@@ -430,21 +373,21 @@ assemble_pEmit<- function(theta) {
         i = ((k1-1)*k1)/2+k2
         pEmit1[1,i] = (1-theta[m,k1])*(1-theta[m,k2])
         pEmit1[2,i] = theta[m,k1]*(1-theta[m,k2]) + theta[m,k2]*(1-theta[m,k1])
-        pEmit1[3,i] = theta[m,k1]*theta[m,k2] 
+        pEmit1[3,i] = theta[m,k1]*theta[m,k2]
       }
     }
     pEmit[m,,] = pEmit1
-    
+
     # Enforce normalization
     pEmit1Sums = colSums(pEmit1)
-    stopifnot(abs(pEmit1Sums-1)<1e-6)
+    #stopifnot(abs(pEmit1Sums-1)<1e-6)
     pEmit[m,,] = sweep(pEmit[m,,],2,pEmit1Sums,`/`)
   }
   return(pEmit)
 }
 
 #' Compute the haplotype emission distributions based on the fastPHASE HMM
-#'  
+#'
 #' @rdname assemble_pEmit_phased
 #' @keywords internal
 assemble_pEmit_phased<- function(theta) {
@@ -459,7 +402,7 @@ assemble_pEmit_phased<- function(theta) {
 }
 
 #' Compute the genotype initial distributions based on the fastPHASE HMM
-#'  
+#'
 #' @rdname assemble_pInit
 #' @keywords internal
 assemble_pInit <- function(alpha) {
@@ -478,25 +421,25 @@ assemble_pInit <- function(alpha) {
   }
   # Enforce normalization
   pInitSum = sum(pInit)
-  stopifnot(abs(pInitSum-1)<1e-4)
+  #stopifnot(abs(pInitSum-1)<1e-4)
   pInit = pInit/pInitSum
-  
+
   return(pInit)
 }
 
 #' Compute the haplotype initial distributions based on the fastPHASE HMM
-#'  
+#'
 #' @rdname assemble_pInit_phased
 #' @keywords internal
 assemble_pInit_phased <- function(alpha) {
   p = dim(alpha)[1]
   K = dim(alpha)[2]
   pInit = as.numeric(alpha[1,])
-  
+
   # Enforce normalization
   pInitSum = sum(pInit)
-  stopifnot(abs(pInitSum-1)<1e-4)
+  #stopifnot(abs(pInitSum-1)<1e-4)
   pInit = pInit/pInitSum
-  
+
   return(pInit)
-}
\ No newline at end of file
+}
