@@ -1,7 +1,7 @@
 /*
   This file is part of SNPknock.
 
-    Copyright (C) 2017-2018 Matteo Sesia
+    Copyright (C) 2017-2019 Matteo Sesia
 
     SNPknock is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,10 +22,8 @@
 
 #include "genotypes.h"
 
-using namespace knockoffs;
-
-GenotypeModel::GenotypeModel(const std::vector<double> & r, const matrix & alpha, const matrix & _theta,
-                             int seed){
+GroupGenotypes::GroupGenotypes(const std::vector<double> & r, const matrix & alpha, const matrix & _theta,
+                               const std::vector<int> groups, int seed){
   // Store input parameters
   theta = _theta;
   p = alpha.size();
@@ -64,18 +62,24 @@ GenotypeModel::GenotypeModel(const std::vector<double> & r, const matrix & alpha
       table[m][1] = j;
     }
   }
-  C = std::vector<double>(nStates,0);
-
-  Hk = std::vector<int> (p);
-  Xk = std::vector<int> (p);
+  Xk = std::vector<int> (p,0);
+  indices = std::vector<int>(K);
   // Knockoffs for Markov chains
+  C = std::vector<double>(nStates,0.0);
+  D = std::vector<double>(K,0.0);
+  Hk = std::vector<int> (p,0);
   Z = std::vector<double> (nStates);
   Z_old = std::vector<double> (nStates,1);
-  indices = std::vector<int>(K);
-  C_sums_partial = std::vector<double>(K);
+
+  // Process group membership
+  nGroups = groups.back()+1;
+  elements.resize(nGroups);
+  for(int i=0; i<p; i++) {
+    elements[groups[i]].push_back(i);
+  }
 }
 
-int GenotypeModel::pair_to_index(int k, int l) {
+int GroupGenotypes::pair_to_index(int k, int l) {
   // Make sure that k >= l
   if(k<l) {
     int tmp = l;
@@ -85,7 +89,7 @@ int GenotypeModel::pair_to_index(int k, int l) {
   return((k*(k+1))/2+l);
 }
 
-std::vector<int> GenotypeModel::single_to_indices(int j) {
+std::vector<int> GroupGenotypes::single_to_indices(int j) {
   std::vector<int> indices(K);
   for(int i=0; i<K; i++) {
     indices[i] = pair_to_index(i,j);
@@ -93,17 +97,17 @@ std::vector<int> GenotypeModel::single_to_indices(int j) {
   return(indices);
 }
 
-double GenotypeModel::emission_prob(int j, int x, int m) {
+double GroupGenotypes::emission_prob(int j, int x, int m) {
   return emission_prob(j, x, table[m][0], table[m][1]);
 }
 
-double GenotypeModel::emission_prob(int j, int x, int k, int l) {
-  double prob = 0;
+double GroupGenotypes::emission_prob(int j, int x, int k, int l) {
+  double prob = 0.0;
   if(x==0) {
-    prob = (1-theta[j][k])*(1-theta[j][l]);
+    prob = (1.0-theta[j][k])*(1.0-theta[j][l]);
   }
   else if(x==1) {
-    prob = theta[j][k]*(1-theta[j][l]) + theta[j][l]*(1-theta[j][k]);
+    prob = theta[j][k]*(1.0-theta[j][l]) + theta[j][l]*(1.0-theta[j][k]);
   }
   else if(x==2) {
     prob = theta[j][k]*theta[j][l];
@@ -111,7 +115,41 @@ double GenotypeModel::emission_prob(int j, int x, int k, int l) {
   return(prob);
 }
 
-void GenotypeModel::sampleViterbi(const std::vector<int> & X) {
+double GroupGenotypes::Q_bar(int j, int k, int l) {
+  int k1 = table[k][0];
+  int k2 = table[k][1];
+  int l1 = table[l][0];
+  int l2 = table[l][1];
+  double q;
+
+  if(j==0) {
+    q = (2.0-(k1==k2)) * a[j][k1] * a[j][k2];
+  }
+  else {
+    q  = (a[j][k1] + b[j]*(k1==l1)) * (a[j][k2] + b[j]*(k2==l2));
+    if(k1!=k2){
+      q += (a[j][k2] + b[j]*(k2==l1)) * (a[j][k1] + b[j]*(k1==l2));
+    }
+  }
+  return(q);
+}
+
+double GroupGenotypes::V_bar(int k, int l, const std::vector<double>& v, const std::vector<double>& w, double u){
+  // FIX ME: update this with correct recursion formulae
+  int k1 = table[k][0];
+  int k2 = table[k][1];
+  int l1 = table[l][0];
+  int l2 = table[l][1];
+  double V;
+
+  V  = v[k] * (2.0-(k1==k2));
+  V += ( w[k1]*((k2==l1)+(k2==l2)) + w[k2]*((k1==l1)+(k1==l2)) ) / (1.0+(k1==k2));
+  V += u * (k==l);
+
+  return(V);
+}
+
+void GroupGenotypes::sampleViterbi(const std::vector<int> & X) {
   // Compute backward weights
   std::fill(beta[p-1].begin(), beta[p-1].end(), 1.0);
 
@@ -205,118 +243,165 @@ void GenotypeModel::sampleViterbi(const std::vector<int> & X) {
   }
 }
 
-void GenotypeModel::knockoffMC(const std::vector<int> & H) {
+void GroupGenotypes::knockoffMC(const std::vector<int> & H) {
   std::fill(Z_old.begin(), Z_old.end(), 1.0);
-  double weights_sum;
-  for(int j=0; j<p; j++) {
-    std::fill(weights.begin(), weights.end(), 1.0);
-    std::fill(Z.begin(), Z.end(), 0.0);
-    std::fill(C.begin(), C.end(), 1.0);
-    std::fill(C_sums_partial.begin(), C_sums_partial.end(), 0.0);
-    weights_sum = 0;
+  double Z_min = 1.0e-10;
 
-    if(j<p-1) {
-      // Precompute vector C
-      double C_sum = 0;
+  for(int g=0; g<nGroups; g++) {
+    int groupSize = elements[g].size();
+
+    // Compute v,w,u
+
+    // FIXME: update this with correct recursion formulae
+    std::vector<double> u;
+    matrix v,w;
+    u = std::vector<double>(groupSize, 0);
+    v = matrix(groupSize, std::vector<double> (nStates,0.0));
+    w = matrix(groupSize, std::vector<double> (K,0.0));
+
+    for(int j=groupSize-1; j>=0; j--) {
+      // Update u
+      if(j == groupSize-1) {
+        if(g < nGroups-1) {
+          u[j] = b[elements[g+1][0]] * b[elements[g+1][0]];
+        }
+        else {
+          u[j] = 0.0;
+        }
+      }
+      else {
+        u[j] = u[j+1] * b[elements[g][j+1]] * b[elements[g][j+1]];
+      }
       for(int k1=0; k1<K; k1++) {
-        for(int k2=0; k2<=k1; k2++) {
+        for(int k2=0; k2<K; k2++) {
           int k12 = pair_to_index(k1,k2);
-          if(j==0) {
-            C[k12] = a[j][k1] * a[j][k2];
-            if(k1!=k2) {
-              C[k12] += a[j][k1] * a[j][k2];
-            }
-          }
-          else {
-            int l1  = table[H[j-1]][0];
-            int l2  = table[H[j-1]][1];
-            int l1k = table[Hk[j-1]][0];
-            int l2k = table[Hk[j-1]][1];
-            if(k1==k2) {
-              C[k12]  = (a[j][k1]+b[j]*(k1==l1)) * (a[j][k2]+b[j]*(k2==l2));
-              C[k12] *= (a[j][k1]+b[j]*(k1==l1k)) * (a[j][k2]+b[j]*(k2==l2k));
+          if(k2<=k1) {
+            // Update v
+            if(j == groupSize-1) {
+              if(g < nGroups-1) {
+                v[j][k12] = a[elements[g+1][0]][k1] * a[elements[g+1][0]][k2];
+              }
+              else {
+                v[j][k12] = 1.0;
+              }
             }
             else {
-              double tmp1 = (a[j][k1]+b[j]*(k1==l1)) * (a[j][k2]+b[j]*(k2==l2));
-              tmp1 += (a[j][k2]+b[j]*(k2==l1)) * (a[j][k1]+b[j]*(k1==l2));
-              double tmp2 = (a[j][k1]+b[j]*(k1==l1k)) * (a[j][k2]+b[j]*(k2==l2k));
-              tmp2 += (a[j][k2]+b[j]*(k2==l1k)) * (a[j][k1]+b[j]*(k1==l2k));
-              C[k12] = tmp1 * tmp2;
+              v[j][k12]  = v[j+1][k12];
+              v[j][k12] += u[j+1]*a[elements[g][j+1]][k1]*a[elements[g][j+1]][k2];
+              v[j][k12] += w[j+1][k1]*a[elements[g][j+1]][k2];
+              v[j][k12] += w[j+1][k2]*a[elements[g][j+1]][k1];
             }
           }
-          C[k12] /= Z_old[k12];
-          // Precompute sum of C
-          C_sum += C[k12];
-          // Precompute partial sums of C
-          C_sums_partial[k1] += C[k12];
-          C_sums_partial[k2] += C[k12];
+        }
+        // Update w
+        if(j == groupSize-1) {
+          if(g < nGroups-1) {
+            w[j][k1] = b[elements[g+1][0]] * a[elements[g+1][0]][k1];
+          }
+          else {
+            w[j][k1] = 0.0;
+          }
+        }
+        else {
+          w[j][k1]  = w[j+1][k1] * b[elements[g][j+1]];
+          w[j][k1] += u[j+1] * a[elements[g][j+1]][k1] * b[elements[g][j+1]];
         }
       }
+    }
 
-      // Compute partition function
+    std::fill(C.begin(), C.end(), 0.0);
+    std::fill(D.begin(), D.end(), 0.0);
+    std::fill(Z.begin(), Z.end(), 0.0);
+    if(g < nGroups-1) {
+      // Precompute C and D for partition function
+      int j0 = elements[g][0];
+      double C_sum = 0.0;
       for(int k1=0; k1<K; k1++) {
         for(int k2=0; k2<=k1; k2++) {
           int k12 = pair_to_index(k1,k2);
-          Z[k12]  = a[j+1][k1] * a[j+1][k2] * C_sum;
-          Z[k12] += b[j+1] * b[j+1] * C[k12];
-          Z[k12] += a[j+1][k1] * b[j+1] * C_sums_partial[k2];
-          if(k1!=k2) {
-            Z[k12] += a[j+1][k1] * a[j+1][k2] * C_sum;
-            Z[k12] += a[j+1][k2] * b[j+1] * C_sums_partial[k1];
-          }
-        }
-      }
-    }
-
-    // Compute weights
-    for(int k1=0; k1<K; k1++) {
-      for(int k2=0; k2<=k1; k2++) {
-        int k12 = pair_to_index(k1,k2);
-        if(j>0) {
-          int l1  = table[H[j-1]][0];
-          int l2  = table[H[j-1]][1];
-          int l1k = table[Hk[j-1]][0];
-          int l2k = table[Hk[j-1]][1];
-          double weight_left = 1.0;
-          if(k1==k2) {
-            weight_left = (a[j][k1]+b[j]*(k1==l1)) * (a[j][k2]+b[j]*(k2==l2));
-            weight_left *= (a[j][k1]+b[j]*(k1==l1k)) * (a[j][k2]+b[j]*(k2==l2k));
+          if(j0==0) {
+            C[k12] = Q_bar(j0, k12, k12);
           }
           else {
-            double tmp1 = (a[j][k1]+b[j]*(k1==l1)) * (a[j][k2]+b[j]*(k2==l2));
-            tmp1 += (a[j][k2]+b[j]*(k2==l1)) * (a[j][k1]+b[j]*(k1==l2));
-            double tmp2 = (a[j][k1]+b[j]*(k1==l1k)) * (a[j][k2]+b[j]*(k2==l2k));
-            tmp2 += (a[j][k2]+b[j]*(k2==l1k)) * (a[j][k1]+b[j]*(k1==l2k));
-            weight_left = tmp1 * tmp2;
-          }          
-          weights[k12] *= weight_left;
-        }
-        if(j<p-1) {
-          int l1  = table[H[j+1]][0];
-          int l2  = table[H[j+1]][1];
-          double weight_right = (a[j+1][l1]+b[j+1]*(k1==l1)) * (a[j+1][l2]+b[j+1]*(k2==l2));
-          if(l1!=l2) {
-            weight_right += (a[j+1][l1]+b[j+1]*(k2==l1)) * (a[j+1][l2]+b[j+1]*(k1==l2));
+            C[k12] = Q_bar(j0, k12, H[j0-1]) * Q_bar(j0, k12, Hk[j0-1]) / Z_old[k12];
           }
-          weights[k12] *= weight_right;
+          D[k1] += C[k12];
+          D[k2] += C[k12];
+          C_sum += C[k12];
         }
-        weights[k12] /= Z_old[k12];
-        Z_old[k12] = Z[k12];
-        weights_sum += weights[k12];
       }
+
+      // Compute partition function (FIXME: update this)
+      double Z_norm = 0;
+      for(int k1=0; k1<K; k1++) {
+        for(int k2=0; k2<=k1; k2++) {
+          int k12 = pair_to_index(k1,k2);
+          Z[k12]  = (2.0-(k1==k2)) * v[0][k12] * C_sum;
+          Z[k12] += (w[0][k1]*D[k2] + w[0][k2]*D[k1]) / (1.0 + (k1==k2));
+          Z[k12] += u[0] * C[k12];
+          Z_norm += Z[k12];
+        }
+      }
+
+      // Normalize partition function and make sure we avoid division by zero
+      for(int k1=0; k1<K; k1++) {
+        for(int k2=0; k2<=k1; k2++) {
+          int k12 = pair_to_index(k1,k2);
+          Z[k12] /= Z_norm;
+          if(Z[k12] < Z_min) {
+            Z[k12] = Z_min;
+          }
+        }
+      }
+
     }
 
-    // Normalize weights
-    for(int k12=0; k12<nStates; k12++) {
-      weights[k12] /= weights_sum;
+    // Compute sampling weights
+    for(int j=0; j<groupSize; j++) {
+      std::fill(weights.begin(), weights.end(), 1.0);
+      double weights_sum = 0;
+      for(int k=0; k<nStates; k++) {
+        // Compute first part of transition matrix
+        if(j==0) {
+          if( g==0 ) {
+            weights[k] = Q_bar(elements[g][0], k, 0);
+          }
+          else {
+            int H0  = H[elements[g-1].back()];
+            int H0k = Hk[elements[g-1].back()];
+            weights[k] = Q_bar(elements[g][0], k, H0) * Q_bar(elements[g][0], k, H0k) / Z_old[k];
+          }
+        }
+        else {
+          weights[k] = Q_bar(elements[g][j], k, Hk[elements[g][j-1]]);
+        }
+
+        // Compute second part of transition matrix
+        if(g < nGroups-1) {
+          weights[k] *= V_bar(H[elements[g+1][0]], k, v[j], w[j], u[j]);
+        }
+        else {
+          weights[k] *= V_bar(0, k, v[j], w[j], u[j]);
+        }
+
+        weights_sum += weights[k];
+      }
+
+      // Normalize weights
+      for(int k=0; k<nStates; k++) {
+        weights[k] /= weights_sum;
+      }
+
+      Hk[elements[g][j]] = weighted_choice(dis(gen2),weights);
     }
 
-    // Sample knockoff copy at this site
-    Hk[j] = weighted_choice(dis(gen2),weights);
+    for(int k=0; k<nStates; k++) {
+      Z_old[k] = Z[k];
+    }
   }
 }
 
-void GenotypeModel::emission(const std::vector<int> & Hk) {
+void GroupGenotypes::emission(const std::vector<int> & Hk) {
   std::vector<double> weights3(3,1.0);
   for(int j=0; j<p; j++) {
     weights3[0] = emission_prob(j, 0, Hk[j]);
@@ -326,14 +411,14 @@ void GenotypeModel::emission(const std::vector<int> & Hk) {
   }
 }
 
-std::vector<int> GenotypeModel::sample(const std::vector<int> & X) {
+std::vector<int> GroupGenotypes::sample(const std::vector<int> & X) {
   sampleViterbi(X);
   knockoffMC(H);
   emission(Hk);
   return(Xk);
 }
 
-imatrix GenotypeModel::sample(const imatrix & X) {
+imatrix GroupGenotypes::sample(const std::vector<std::vector<int> > & X) {
   unsigned int n = X.size();
   imatrix XkMatrix(n, std::vector<int>(p));
   for(unsigned int i=0; i<X.size(); i++) {
