@@ -1,7 +1,7 @@
 /*
   This file is part of SNPknock.
 
-    Copyright (C) 2017-2018 Matteo Sesia
+    Copyright (C) 2017-2019 Matteo Sesia
 
     SNPknock is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,14 +22,13 @@
 
 #include "haplotypes.h"
 
-using namespace knockoffs;
-
-HaplotypeModel::HaplotypeModel(const std::vector<double> & r, const matrix & alpha, const matrix & _theta,
-                               int seed){
+GroupHaplotypes::GroupHaplotypes(const std::vector<double> & r, const matrix & alpha, const matrix & _theta,
+                               const std::vector<int> groups, int seed){
   // Store input parameters
   theta = _theta;
   p = alpha.size();
   nStates = alpha[0].size();
+
   // Process MC parameters
   a = matrix(p, std::vector<double> (nStates));
   b = std::vector<double>(p, 0);
@@ -53,15 +52,23 @@ HaplotypeModel::HaplotypeModel(const std::vector<double> & r, const matrix & alp
   // Initialize variables
   weights = std::vector<double> (nStates);
   beta = matrix(p, std::vector<double> (nStates));
-  H = std::vector<int> (p);
-  Hk = std::vector<int> (p);
-  Xk = std::vector<int> (p);
+  H = std::vector<int> (p,0);
+  Hk = std::vector<int> (p,0);
+  Xk = std::vector<int> (p,0);
   // Knockoffs for Markov chains
-  Z = std::vector<double> (nStates);
-  Z_old = std::vector<double> (nStates);
+  Z = std::vector<double> (nStates,1.0);
+  Z_old = std::vector<double> (nStates,1.0);
+
+  // Process group membership
+  nGroups = groups.back()+1;
+  elements.resize(nGroups);
+  for(int i=0; i<p; i++) {
+    elements[groups[i]].push_back(i);
+  }
+
 }
 
-void HaplotypeModel::sampleViterbi(const std::vector<int> & X) {
+void GroupHaplotypes::sampleViterbi(const std::vector<int> & X) {
   // Compute backward weights
   std::fill(beta[p-1].begin(), beta[p-1].end(), 1.0);
   for(int j=p-2; j>=0; j--) {
@@ -82,7 +89,6 @@ void HaplotypeModel::sampleViterbi(const std::vector<int> & X) {
       else {
         beta[j][k] = beta_const + b[j+1] * (1.0-theta[j+1][k]) * beta[j+1][k];
       }
-
       betaSum += beta[j][k];
     }
     for(int k=0; k<nStates; k++) {
@@ -124,66 +130,147 @@ void HaplotypeModel::sampleViterbi(const std::vector<int> & X) {
   }
 }
 
-void HaplotypeModel::knockoffMC(const std::vector<int> & H) {
+void GroupHaplotypes::knockoffMC(const std::vector<int> & H) {
   std::fill(Z_old.begin(), Z_old.end(), 1.0);
-  double weights_sum, Z_sum;
+  double Z_min = 1.0e-10;
 
-  for(int j=0; j<p; j++) {
-    std::fill(weights.begin(), weights.end(), 1.0);
-    std::fill(Z.begin(), Z.end(), 0.0);
-    weights_sum = 0;
-    Z_sum = 0;
-
-    // Precompute sum for partition function
-    for(int k=0; k<nStates; k++) {
-      if( j==0 ) {
-        Z_sum += a[j][k];
+  for(int g=0; g<nGroups; g++) {
+    // Compute vstar
+    int groupSize = elements[g].size();
+    std::vector<double> vstar(groupSize,0.0);
+    for(int j=groupSize-1; j>=0; j--) {
+      if(j < groupSize-1) {
+        vstar[j] = vstar[j+1] * b[elements[g][j+1]];
       }
       else {
-        Z_sum += (a[j][k] + (double)(k==H[j-1])) * (a[j][k] + (double)(k==Hk[j-1])) / Z_old[k];
+        if(g < nGroups-1) {
+          vstar[j] = b[elements[g+1][0]];
+        }
+        else {
+          vstar[j] = 0.0;
+        }
+      }
+    }
+
+    // Compute vbar matrix
+    matrix vbar;
+    vbar = matrix(groupSize, std::vector<double> (nStates,0));
+    for(int j=groupSize-1; j>=0; j--) {
+      double sum_a = 0;
+      if(j < groupSize-1) {
+        for(int l=0; l<nStates; l++) {
+          sum_a += a[elements[g][j+1]][l];
+        }
+      }
+      for(int z=0; z<nStates; z++) {
+        if(j < groupSize-1) {
+          vbar[j][z] = vbar[j+1][z] * (sum_a + b[elements[g][j+1]]);
+          vbar[j][z] += vstar[j+1] * a[elements[g][j+1]][z];
+        }
+        else {
+          if(g < nGroups-1) {
+            vbar[j][z] = a[elements[g+1][0]][z];
+          }
+          else {
+            vbar[j][z] = 1.0;
+          }
+        }
+      }
+    }
+
+    // Precompute sum for partition function
+    double Z_sum = 0;
+    for(int k=0; k<nStates; k++) {
+      if( g==0 ) {
+        Z_sum += a[elements[g][0]][k];
+      }
+      else {
+        int H0 = H[elements[g-1].back()];
+        int H0k = Hk[elements[g-1].back()];
+        double tmp = (a[elements[g][0]][k] + b[elements[g][0]]*(double)(k==H0));
+        tmp  = tmp * (a[elements[g][0]][k] + b[elements[g][0]]*(double)(k==H0k));
+        Z_sum += tmp / Z_old[k];
       }
     }
 
     // Compute partition function
+    std::fill(Z.begin(), Z.end(), 0.0);
     for(int k=0; k<nStates; k++) {
-      if(j<p-1) {
-        Z[k] += a[j+1][k] * Z_sum;
-        if(j==0) {
-          Z[k] += b[j+1] * a[j][k];
+      if(g < nGroups-1) {
+        Z[k] += vbar[0][k] * Z_sum;
+        if(g==0) {
+          Z[k] += vstar[0] * a[elements[g][0]][k];
         }
         else {
-          Z[k] += b[j+1] * (a[j][k] + (double)(k==H[j-1])) * (a[j][k] + (double)(k==Hk[j-1])) / Z_old[k];
+          int H0 = H[elements[g-1].back()];
+          int H0k = Hk[elements[g-1].back()];
+          double tmp = (a[elements[g][0]][k] + b[elements[g][0]]*(double)(k==H0));
+          tmp  = tmp * (a[elements[g][0]][k] + b[elements[g][0]]*(double)(k==H0k));
+          Z[k] += vstar[0] * tmp / Z_old[k];
         }
       }
     }
 
+    // Normalize partition function and make sure we avoid division by zero
+    double Z_norm = 0;
+    for(int k=0; k<nStates; k++) {
+      if(Z[k] < Z_min) {
+        Z[k] = Z_min;
+      }
+      Z_norm += Z[k];
+    }
+    for(int k=0; k<nStates; k++) {
+      Z[k] /= Z_norm;
+      if(Z[k] < Z_min) {
+        Z[k] = Z_min;
+      }
+    }
+
     // Compute sampling weights
+    for(int j=0; j<groupSize; j++) {
+      std::fill(weights.begin(), weights.end(), 1.0);
+      double weights_sum = 0;
+      for(int k=0; k<nStates; k++) {
+        if(j>0) {
+          int H0k = Hk[elements[g][j-1]];
+          weights[k] *= (a[elements[g][j]][k]+b[elements[g][j]]*(double)(k==H0k));
+        }
+        else {
+          if( g==0 ) {
+            weights[k] *= a[elements[g][0]][k];
+          }
+          else {
+            int H0 = H[elements[g-1].back()];
+            int H0k = Hk[elements[g-1].back()];
+            weights[k] *= (a[elements[g][0]][k]+b[elements[g][0]]*(double)(k==H0));
+            weights[k] *= (a[elements[g][0]][k]+b[elements[g][0]]*(double)(k==H0k));
+          }
+          weights[k] /= Z_old[k];
+        }
+        if(g == nGroups-1) {
+          weights[k] *= (vbar[j][0]);
+        }
+        else {
+          int H1 = H[elements[g+1][0]];
+          weights[k] *= (vbar[j][H1] + vstar[j] * (double)(k==H1));
+        }
+        weights_sum += weights[k];
+      }
+
+      // Normalize weights
+      for(int k=0; k<nStates; k++) {
+        weights[k] /= weights_sum;
+      }
+      Hk[elements[g][j]] = weighted_choice(dis(gen2),weights);
+    }
+
     for(int k=0; k<nStates; k++) {
-      if(j < p-1) {
-        weights[k] *= (a[j+1][H[j+1]]+b[j+1]*(double)(k==H[j+1]));
-      }
-      if(j==0) {
-        weights[k] *= a[j][k];
-      }
-      else {
-        weights[k] *= (a[j][k]+b[j]*(double)(k==H[j-1]));
-        weights[k] *= (a[j][k]+b[j]*(double)(k==Hk[j-1]));
-      }
-      weights[k] /= Z_old[k];
       Z_old[k] = Z[k];
-      weights_sum += weights[k];
     }
-
-    // Normalize weights
-    for(int k=0; k<nStates; k++) {
-      weights[k] /= weights_sum;
-    }
-
-    Hk[j] = weighted_choice(dis(gen2),weights);
   }
 }
 
-void HaplotypeModel::emission(const std::vector<int> & Hk) {
+void GroupHaplotypes::emission(const std::vector<int> & Hk) {
   std::vector<double> weights2(2,1.0);
   // A little slower, but random seeds are compatible with older code
   for(int j=0; j<p; j++) {
@@ -193,14 +280,14 @@ void HaplotypeModel::emission(const std::vector<int> & Hk) {
   }
 }
 
-std::vector<int> HaplotypeModel::sample(const std::vector<int> & X) {
+std::vector<int> GroupHaplotypes::sample(const std::vector<int> & X) {
   sampleViterbi(X);
   knockoffMC(H);
   emission(Hk);
   return(Xk);
 }
 
-imatrix HaplotypeModel::sample(const std::vector<std::vector<int> > & X) {
+imatrix GroupHaplotypes::sample(const std::vector<std::vector<int> > & X) {
   unsigned int n = X.size();
   imatrix XkMatrix(n, std::vector<int>(p));
   for(unsigned int i=0; i<X.size(); i++) {
